/*-------------------------------------------------------------------------
 *
 *  IniFile.cc :
 *    Defines the TIniFile class, based on glib's key-file mechanism
 *
 *  $Id: IniFile.cc 419 2007-06-03 14:53:21Z rob $
 --------------------------------------------------------------------------*/


#include "IniFile.h"

TIniFile::TIniFile ( std::string filename ) 
{
    gchar * full_path;
    GKeyFileFlags Options;
    Options = (GKeyFileFlags)(G_KEY_FILE_KEEP_COMMENTS|G_KEY_FILE_KEEP_TRANSLATIONS);

    pData = g_key_file_new();

    if ( g_key_file_load_from_file ( pData, filename.c_str(), 
            Options, NULL ) )
    {
        FileName = filename;
        Dirty = false;
    }
    else if ( g_key_file_load_from_data_dirs ( pData, filename.c_str(),
                 &full_path, Options, NULL ) )
    {
        FileName = full_path;
        Dirty = false;
    }
    else if ( g_path_is_absolute ( filename.c_str() ) )
    {
        FileName = filename;
        Dirty = true;       // Should we do this?
    }
    else
    {
        FileName = g_get_user_data_dir();
        g_mkdir_with_parents ( FileName.c_str(), 0755 );
        FileName += "/";
        FileName += filename;
        Dirty = true;
    }
//    printf ( "TIniFile path is %s\n", FileName.c_str() );
}

TIniFile::~TIniFile()
{
    g_key_file_free ( pData );
}

void TIniFile::ReadSection ( const std::string section, 
                             std::vector<std::string> * keys )
{
    gchar ** list;
    gsize len;

    keys->clear();
    list = g_key_file_get_keys ( pData, section.c_str(), &len, NULL );
    if ( list )
        for ( int cc=0; cc<len; cc++ )
        {
            keys->push_back ( std::string ( list[cc] ) );
        }
    g_strfreev ( list );
}

void TIniFile::ReadSections ( std::vector<std::string> * sections )
{
    gchar ** list;
    gsize len;
    
    list = g_key_file_get_groups ( pData, &len );
    sections->clear();
    for ( int cc=0; cc<len; cc++ )
    {
        sections->push_back ( std::string ( list[cc] ) );
    }
    g_strfreev ( list );
}

bool TIniFile::SectionExists ( const std::string section )
{
    return g_key_file_has_group ( pData, section.c_str() );
}

std::string TIniFile::ReadString ( const std::string section, 
                         const std::string key, 
                         const std::string defval )
{
    std::string result = defval;
    gchar * tresult;

    tresult = g_key_file_get_string ( pData, section.c_str(), key.c_str(), NULL );
    if ( tresult )
    {
        result = tresult;
        g_free ( tresult );
    }
    return result;
}

bool TIniFile::ReadBool ( const std::string section, 
                const std::string key, 
                bool defval )
{
    GError * err = NULL;
    bool result;

    result = g_key_file_get_boolean ( pData, section.c_str(), key.c_str(), &err );
    if ( err )
        result = defval;
    return result;
}

int TIniFile::ReadInteger ( const std::string section, 
                const std::string key, 
                int defval )
{
    GError * err = NULL;
    int result;

    result = g_key_file_get_integer ( pData, section.c_str(), key.c_str(), &err );
    if ( err )
        result = defval;
    return result;
}

void TIniFile::WriteString ( const std::string section, 
                             const std::string key, 
                             const std::string value )
{
    g_key_file_set_string ( pData, section.c_str(), key.c_str(), value.c_str() );
    Dirty = true;
}

void TIniFile::WriteBool ( const std::string section, 
                           const std::string key, 
                           bool value )
{
    g_key_file_set_boolean ( pData, section.c_str(), key.c_str(), value );
    Dirty = true;
}

void TIniFile::UpdateFile ( bool force )
{
    gchar * result;
    GError * err = NULL;
    gsize len;

    if ( Dirty || force )
    {
        result = g_key_file_to_data ( pData, &len, &err );
        if ( err == NULL )
        {
            g_file_set_contents ( FileName.c_str(), result, len, &err );
        }
        if ( err )
        {
            fprintf ( stderr, "TIniFile::UpdateFile error '%s'\n", err->message );
        }
    }
}
